@@ -20,6 +20,8 @@
  */
 
 #include <stdlib.h>
+#include <stdio.h>	/* for snprintf */
+#include <string.h>	/* for strdup */
 
 #include <libdivecomputer/units.h>
 
@@ -31,6 +33,10 @@
 
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &mares_iconhd_parser_vtable)
 
+#define OBJVERSION(major,minor) ( \
+		(((major) & 0xFF) << 8) | \
+		((minor) & 0xFF))
+
 #define SMART      0x000010
 #define SMARTAPNEA 0x010010
 #define ICONHD    0x14
@@ -139,10 +145,12 @@ struct mares_iconhd_parser_t {
 	unsigned int samplesize;
 	unsigned int headersize;
 	unsigned int settings;
+	unsigned int surftime;
 	unsigned int interval;
 	unsigned int samplerate;
 	unsigned int ntanks;
 	unsigned int ngasmixes;
+	unsigned int serial;
 	mares_iconhd_gasmix_t gasmix[NGASMIXES];
 	mares_iconhd_tank_t tank[NTANKS];
 };
@@ -343,6 +351,7 @@ mares_iconhd_cache (mares_iconhd_parser_t *parser)
 	parser->samplesize = samplesize;
 	parser->headersize = headersize;
 	parser->settings = settings;
+	parser->surftime = 3 * 60;
 	parser->interval = interval;
 	parser->samplerate = samplerate;
 	parser->ntanks = ntanks;
@@ -374,7 +383,7 @@ mares_genius_cache (mares_iconhd_parser_t *parser)
 	unsigned int type = array_uint16_le (data);
 	unsigned int minor = data[2];
 	unsigned int major = data[3];
-	if (type != 1 || major != 0 || minor > 1) {
+	if (type != 1 || OBJVERSION(major,minor) > OBJVERSION(1,1)) {
 		ERROR (abstract->context, "Unsupported object type (%u) or version (%u.%u).",
 			type, major, minor);
 		return DC_STATUS_DATAFORMAT;
@@ -389,8 +398,14 @@ mares_genius_cache (mares_iconhd_parser_t *parser)
 		extra = 8;
 	}
 
+	// The Genius header (v1.x) has 10 bytes more at the end.
+	unsigned int more = 0;
+	if (major == 1) {
+		more = 16;
+	}
+
 	// Get the header size.
-	unsigned int headersize = 0xB8 + extra;
+	unsigned int headersize = 0xB8 + extra + more;
 	if (headersize > size) {
 		ERROR (abstract->context, "Buffer overflow detected!");
 		return DC_STATUS_DATAFORMAT;
@@ -415,6 +430,22 @@ mares_genius_cache (mares_iconhd_parser_t *parser)
 		return DC_STATUS_DATAFORMAT;
 	}
 
+	// Get the profile type and version.
+	unsigned int profile_type = array_uint16_le (data + headersize);
+	unsigned int profile_minor = data[headersize + 2];
+	unsigned int profile_major = data[headersize + 3];
+
+	// Get the surface timeout setting (in minutes).
+	// For older firmware versions the value is hardcoded to 3 minutes, but
+	// starting with the newer v01.02.00 firmware the value is configurable and
+	// stored in the settings. To detect whether the setting is available, we
+	// need to check the profile version instead of the header version.
+	unsigned int surftime = 3;
+	if (profile_type == 0 &&
+		OBJVERSION(profile_major,profile_minor) >= OBJVERSION(1,0)) {
+		surftime = (settings >> 13) & 0x3F;
+	}
+
 	// Gas mixes and tanks.
 	unsigned int ntanks = 0;
 	unsigned int ngasmixes = 0;
@@ -466,6 +497,7 @@ mares_genius_cache (mares_iconhd_parser_t *parser)
 	parser->samplesize = samplesize;
 	parser->headersize = headersize;
 	parser->settings = settings;
+	parser->surftime = surftime * 60;
 	parser->interval = 5;
 	parser->samplerate = 1;
 	parser->ntanks = ntanks;
@@ -496,7 +528,7 @@ mares_iconhd_parser_cache (mares_iconhd_parser_t *parser)
 }
 
 dc_status_t
-mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model, unsigned int serial)
 {
 	mares_iconhd_parser_t *parser = NULL;
 
@@ -519,10 +551,12 @@ mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned i
 	parser->samplesize = 0;
 	parser->headersize = 0;
 	parser->settings = 0;
+	parser->surftime = 0;
 	parser->interval = 0;
 	parser->samplerate = 0;
 	parser->ntanks = 0;
 	parser->ngasmixes = 0;
+	parser->serial = serial;
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
@@ -553,6 +587,7 @@ mares_iconhd_parser_set_data (dc_parser_t *abstract, const unsigned char *data,
 	parser->samplesize = 0;
 	parser->headersize = 0;
 	parser->settings = 0;
+	parser->surftime = 0;
 	parser->interval = 0;
 	parser->samplerate = 0;
 	parser->ntanks = 0;
@@ -625,6 +660,7 @@ mares_iconhd_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime
 	return DC_STATUS_SUCCESS;
 }
 
+#define BUFLEN 16
 
 static dc_status_t
 mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
@@ -657,12 +693,14 @@ mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 	dc_tank_t *tank = (dc_tank_t *) value;
 	dc_salinity_t *water = (dc_salinity_t *) value;
+	dc_field_string_t *string = (dc_field_string_t *) value;
+	char buf[BUFLEN];
 
 	if (value) {
 		switch (type) {
 		case DC_FIELD_DIVETIME:
 			if (parser->model == GENIUS || parser->model == HORIZON) {
-				*((unsigned int *) value) = parser->nsamples * parser->interval;
+				*((unsigned int *) value) = parser->nsamples * parser->interval - parser->surftime;
 			} else if (parser->model == SMARTAPNEA) {
 				*((unsigned int *) value) = array_uint16_le (p + 0x24);
 			} else if (parser->mode == ICONHD_FREEDIVE) {
@@ -674,7 +712,7 @@ mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 				}
 				*((unsigned int *) value) = divetime;
 			} else {
-				*((unsigned int *) value) = parser->nsamples * parser->interval;
+				*((unsigned int *) value) = parser->nsamples * parser->interval - parser->surftime;
 			}
 			break;
 		case DC_FIELD_MAXDEPTH:
@@ -824,6 +862,17 @@ mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 				}
 			}
 			break;
+		case DC_FIELD_STRING:
+			switch(flags) {
+			case 0: /* serial */
+				string->desc = "Serial";
+				snprintf(buf, BUFLEN, "%u", parser->serial);
+				break;
+			default:
+				return DC_STATUS_UNSUPPORTED;
+			}
+			string->value = strdup(buf);
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -869,7 +918,9 @@ mares_iconhd_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 		unsigned int type = array_uint16_le (data);
 		unsigned int minor = data[2];
 		unsigned int major = data[3];
-		if (type > 1 || major != 0 || minor != 2) {
+		if (type > 1 ||
+			(type == 0 && OBJVERSION(major,minor) > OBJVERSION(1,0)) ||
+			(type == 1 && OBJVERSION(major,minor) > OBJVERSION(0,2))) {
 			ERROR (abstract->context, "Unsupported object type (%u) or version (%u.%u).",
 				type, major, minor);
 			return DC_STATUS_DATAFORMAT;
