@@ -62,8 +62,8 @@
 #define HEADER  1
 #define PROFILE 2
 
-#define FRESH 1.000
-#define SALT  1.025
+#define FRESH 1000.0
+#define SALT  1025.0
 
 #define FREEDIVE  0x00000080
 #define GAUGE     0x00001000
@@ -789,7 +789,7 @@ uwatec_smart_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 	const uwatec_smart_header_info_t *table = parser->header;
 	const unsigned char *data = abstract->data;
 
-	double salinity = (parser->watertype == DC_WATER_SALT ? SALT : FRESH);
+	double density = (parser->watertype == DC_WATER_SALT ? SALT : FRESH);
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 	dc_tank_t *tank = (dc_tank_t *) value;
@@ -801,7 +801,7 @@ uwatec_smart_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 			*((unsigned int *) value) = array_uint16_le (data + table->divetime) * 60;
 			break;
 		case DC_FIELD_MAXDEPTH:
-			*((double *) value) = array_uint16_le (data + table->maxdepth) / 100.0 / salinity;
+			*((double *) value) = array_uint16_le (data + table->maxdepth) * (BAR / 1000.0) / (density * GRAVITY);
 			break;
 		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = parser->ngasmixes;
@@ -844,7 +844,7 @@ uwatec_smart_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 			if (table->settings == UNSUPPORTED)
 				return DC_STATUS_UNSUPPORTED;
 			water->type = parser->watertype;
-			water->density = salinity * 1000.0;
+			water->density = density;
 			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
@@ -940,9 +940,9 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 	unsigned int rbt = 99;
 	unsigned int tank = 0;
 	unsigned int gasmix = 0;
-	double depth = 0, depth_calibration = 0;
-	double temperature = 0;
-	double pressure = 0;
+	unsigned int depth = 0, depth_calibration = 0;
+	int temperature = 0;
+	unsigned int pressure = 0;
 	unsigned int heartrate = 0;
 	unsigned int bearing = 0;
 	unsigned int bookmark = 0;
@@ -950,7 +950,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 	// Previous gas mix - initialize with impossible value
 	unsigned int gasmix_previous = 0xFFFFFFFF;
 
-	double salinity = (parser->watertype == DC_WATER_SALT ? SALT : FRESH);
+	double density = (parser->watertype == DC_WATER_SALT ? SALT : FRESH);
 
 	unsigned int interval = 4;
 	if (parser->divemode == DC_DIVEMODE_FREEDIVE) {
@@ -1026,8 +1026,8 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 		const uwatec_smart_event_info_t *events = NULL;
 		switch (table[id].type) {
 		case PRESSURE_DEPTH:
-			pressure += ((signed char) ((svalue >> NBITS) & 0xFF)) / 4.0;
-			depth += ((signed char) (svalue & 0xFF)) / 50.0;
+			pressure += (signed char) ((svalue >> NBITS) & 0xFF);
+			depth += (signed char) (svalue & 0xFF);
 			complete = 1;
 			break;
 		case RBT:
@@ -1040,37 +1040,37 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 			break;
 		case TEMPERATURE:
 			if (table[id].absolute) {
-				temperature = svalue / 2.5;
+				temperature = svalue;
 				have_temperature = 1;
 			} else {
-				temperature += svalue / 2.5;
+				temperature += svalue;
 			}
 			break;
 		case PRESSURE:
 			if (table[id].absolute) {
 				if (parser->trimix) {
 					tank = (value & 0xF000) >> 12;
-					pressure = (value & 0x0FFF) / 4.0;
+					pressure = (value & 0x0FFF);
 				} else {
 					tank = table[id].index;
-					pressure = value / 4.0;
+					pressure = value;
 				}
 				have_pressure = 1;
 				gasmix = tank;
 			} else {
-				pressure += svalue / 4.0;
+				pressure += svalue;
 			}
 			break;
 		case DEPTH:
 			if (table[id].absolute) {
-				depth = value / 50.0;
+				depth = value;
 				if (!calibrated) {
 					calibrated = 1;
 					depth_calibration = depth;
 				}
 				have_depth = 1;
 			} else {
-				depth += svalue / 50.0;
+				depth += svalue;
 			}
 			complete = 1;
 			break;
@@ -1195,7 +1195,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 			}
 
 			if (have_temperature) {
-				sample.temperature = temperature;
+				sample.temperature = temperature / 2.5;
 				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 			}
 
@@ -1216,7 +1216,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 				idx = uwatec_smart_find_tank(parser, tank);
 				if (idx < parser->ntanks) {
 					sample.pressure.tank = idx;
-					sample.pressure.value = pressure;
+					sample.pressure.value = pressure / 4.0;
 					if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
 				}
 			}
@@ -1233,7 +1233,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 			}
 
 			if (have_depth) {
-				sample.depth = (depth - depth_calibration) / salinity;
+				sample.depth = (signed int)(depth - depth_calibration) * (2.0 * BAR / 1000.0) / (density * GRAVITY);
 				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 			}
 
